package tutorial.jsf.ajax.component;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.*;

import tutorial.jsf.ajax.AjaxRendererInterface;

/**
 * <b>Created:</b> Nov 3, 2007<br>
 * <b>Title:</b> AjaxComponentRenderer<br>
 * <b>Description:</b><br>
 * <b>Copyright:</b> Copyright (c) 2007<br>
 * <b>Company:</b> Asparity Decision Solutions <br>
 * @author Matt Tyson
 * <p>
 * Renderer for AjaxComponent.
 */

public class AjaxComponentRenderer extends Renderer implements AjaxRendererInterface {
	private static final transient Log log = LogFactory.getLog(tutorial.jsf.ajax.component.AjaxComponentRenderer.class);
	private static final String INPUT_ID = "tutorial.jsf.ajax.component.INPUT";
	private static final String INPUT_NAME = "tutorial.jsf.ajax.component.INPUT"; // This will be the text field key in the request
	private static final String BUTTON_ID = "tutorial.jsf.ajax.component.BUTTON";
	private static final String MESSAGE_DIV_ID = "tutorial.jsf.ajax.component.MESSAGE_DIV";
	private static final String CLIENT_ID = "tutorial.jsf.ajax.component.CLIENT_ID";
	
	/**default empty constructor*/
	public AjaxComponentRenderer() {
	}
	
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException{
		if (log.isTraceEnabled()) { log.trace("begin encodeBegin()"); }
		
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		
		AjaxComponent ajaxComp = (AjaxComponent)component;
		
		ResponseWriter out = context.getResponseWriter();
		
		String clientId = ajaxComp.getClientId(context);
		
		out.startElement("div", ajaxComp);
		out.writeAttribute("id", clientId, null);
		out.writeAttribute("style", "border:solid; width:200; height:200;", null);
		
		out.startElement("div", ajaxComp); // Message div
		out.writeAttribute("id", MESSAGE_DIV_ID, null);
		out.endElement("div"); // Message div
		
		out.startElement("input", ajaxComp);
		out.writeAttribute("type", "text", null);
		out.writeAttribute("id", INPUT_ID, null);
		out.writeAttribute("name", INPUT_NAME, null);
		out.endElement("input");
		
		out.startElement("button", component);
		out.writeAttribute("type", "button", null);
		out.writeAttribute("name", BUTTON_ID, null);
		out.writeAttribute("id", BUTTON_ID, null);
		out.writeAttribute("value", BUTTON_ID, null);
		out.writeText("Ajax It", "null");
		out.endElement("button");
		
		// A hidden field to hold the URL of the server for the ajax request
		out.startElement("input", ajaxComp);
		out.writeAttribute("id", "tutorial.jsf.ajax.component.SERVER", null);
		out.writeAttribute("type", "hidden", null); 
		out.writeAttribute("value", request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + request.getRequestURI(), null);
		out.endElement("input");
		
		// A hidden field to hold the component Client ID
		out.startElement("input", ajaxComp);
		out.writeAttribute("id", CLIENT_ID, null);
		out.writeAttribute("type", "hidden", null); 
		out.writeAttribute("value", clientId, null);
		out.endElement("input");
		
		out.write("AjaxComponent\n");
		out.startElement("script", ajaxComp);
		out.write("dojo.addOnLoad(AjaxComponent.loadComponent());\n"); // Sets up the client side JS
		out.endElement("script");
	}
	
	/**
	 * Handles the request from the button, which was setup above in the encodeBegin() method.
	 */
	public void handleAjaxRequest(FacesContext context, UIComponent component){
		if (log.isInfoEnabled()) { log.info("BEGIN handleAjaxRequest()"); }
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
	
		String textField = request.getParameter(INPUT_NAME);
		String serverContribution = "SERVER RESPONSE: ";
		StringBuffer xml = null;
		
		if (textField == null){
			if (log.isInfoEnabled()) { log.info("No parameter found for text field."); }
		} else {
			if (log.isTraceEnabled()) { log.trace("textField: " + textField); }
			// We now have the new value entered by the user
			// Now we create our XML response
			xml = new StringBuffer("<response>");
			
			xml.append("<message>" + serverContribution + textField + "</message>");
			
			xml.append("<status>OK</status></response>");
		}
		
		if (xml == null){
			if (log.isInfoEnabled()) { log.info("Response is null."); }
			xml = new StringBuffer(this.getErrorString());
		}
		// Now we are ready to send the response
		HttpServletResponse response =
            (HttpServletResponse)context.getExternalContext().getResponse();
		
		response.setContentType("text/xml");
        response.setHeader("Cache-Control", "no-cache");
        
        try {
        	response.getWriter().write(xml.toString());
        	if (log.isInfoEnabled()) { log.info("Response sent: " + xml); }
        } catch (IOException e){
        	if (log.isErrorEnabled()) { log.error("Error writing ajax response.", e); }
        }
        
	}
	
	protected String getErrorString(){
		return new String("<response><message>There was a problem</message><status>ERROR</status></response>");
	}
}
